@@ -1,43 +1,34 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2018 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2019 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
  * http://www.boost.org/LICENSE_1_0.txt
- * https://github.com/dlang/dmd/blob/master/src/identifier.h
+ * https://github.com/dlang/dmd/blob/master/src/dmd/identifier.h
  */
 
-#ifndef DMD_IDENTIFIER_H
-#define DMD_IDENTIFIER_H
-
-#ifdef __DMC__
 #pragma once
-#endif /* __DMC__ */
 
-#include "root.h"
-#include "rmem.h"
-#include "stringtable.h"
+#include "root/dcompat.h"
+#include "root/root.h"
 
 class Identifier : public RootObject
 {
 private:
     int value;
-    const char *string;
-    size_t len;
+    DArray<const char> string;
 
 public:
+    static Identifier* anonymous();
     static Identifier* create(const char *string);
     bool equals(RootObject *o);
     int compare(RootObject *o);
-    void print();
     const char *toChars();
     int getValue() const;
     const char *toHChars2();
-    int dyncast() const;
+    DYNCAST dyncast() const;
 
-    static StringTable stringtable;
-    static StringTable fullPathStringTable;
     static Identifier *generateId(const char *prefix);
     static Identifier *generateId(const char *prefix, size_t i);
     static Identifier *idPool(const char *s, unsigned len);
@@ -48,8 +39,4 @@ public:
     }
 
     static bool isValidIdentifier(const char *p);
-    static Identifier *lookup(const char *s, size_t len);
-    static void initTable();
 };
-
-#endif /* DMD_IDENTIFIER_H */
