@@ -1,19 +1,14 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2018 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2019 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
  * http://www.boost.org/LICENSE_1_0.txt
- * https://github.com/dlang/dmd/blob/master/src/mars.h
+ * https://github.com/dlang/dmd/blob/master/src/dmd/mars.h
  */
 
-#ifndef DMD_MARS_H
-#define DMD_MARS_H
-
-#ifdef __DMC__
 #pragma once
-#endif
 
 /*
 It is very important to use version control macros correctly - the
@@ -64,9 +59,6 @@ the target object file format:
 #include <stdio.h>
 #include <stdint.h>
 #include <stdarg.h>
-#if IN_LLVM
-#include <stddef.h>
-#endif
 
 #ifdef __DMC__
 #ifdef DEBUG
@@ -81,7 +73,7 @@ struct OutBuffer;
 
 #include "globals.h"
 
-#include "ctfloat.h"
+#include "root/ctfloat.h"
 
 #include "complex_t.h"
 
@@ -90,7 +82,7 @@ struct OutBuffer;
 class Dsymbol;
 class Library;
 struct File;
-void obj_start(char *srcfile);
+void obj_start(const char *srcfile);
 void obj_end(Library *library, File *objfile);
 void obj_append(Dsymbol *s);
 void obj_write_deferred(Library *library);
@@ -100,12 +92,15 @@ void readFile(Loc loc, File *f);
 void writeFile(Loc loc, File *f);
 void ensurePathToNameExists(Loc loc, const char *name);
 
-#if IN_LLVM
-int mars_mainBody(Strings &files, Strings &libmodules);
-#endif
-
-const char *importHint(const char *s);
 /// Little helper function for writing out deps.
 void escapePath(OutBuffer *buf, const char *fname);
 
-#endif /* DMD_MARS_H */
+#if IN_LLVM
+struct Param;
+
+int mars_mainBody(Param &params, Strings &files, Strings &libmodules);
+
+void parseTransitionOption(Param &params, const char *name);
+void parsePreviewOption(Param &params, const char *name);
+void parseRevertOption(Param &params, const char *name);
+#endif
