@@ -7,15 +7,12 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "errors.h"
-#include "globals.h"
+#include "dmd/errors.h"
+#include "dmd/globals.h"
 #include "driver/cl_options.h"
 #include "driver/tool.h"
 #include "gen/logger.h"
 #include "llvm/ADT/Triple.h"
-
-#if LDC_LLVM_VER >= 309
-
 #include "llvm/Object/Archive.h"
 #include "llvm/Object/ArchiveWriter.h"
 #include "llvm/Object/MachO.h"
@@ -23,6 +20,7 @@
 #include "llvm/Support/Errc.h"
 #include "llvm/Support/Path.h"
 #include "llvm/Support/raw_ostream.h"
+#include <cstring>
 
 #if LDC_LLVM_VER >= 500
 #include "llvm/ToolDrivers/llvm-lib/LibDriver.h"
@@ -30,8 +28,6 @@
 #include "llvm/LibDriver/LibDriver.h"
 #endif
 
-#include <cstring>
-
 using namespace llvm;
 
 /* Unlike the llvm-lib driver, llvm-ar is not available as library; it's
@@ -257,8 +253,6 @@ int internalLib(ArrayRef<const char *> args) {
 
 } // anonymous namespace
 
-#endif // LDC_LLVM_VER >= 309
-
 ////////////////////////////////////////////////////////////////////////////////
 
 static llvm::cl::opt<std::string> ar("ar", llvm::cl::desc("Archiver"),
@@ -270,11 +264,7 @@ int createStaticLibrary() {
   const bool isTargetMSVC =
       global.params.targetTriple->isWindowsMSVCEnvironment();
 
-#if LDC_LLVM_VER >= 309
   const bool useInternalArchiver = ar.empty();
-#else
-  const bool useInternalArchiver = false;
-#endif
 
   // find archiver
   std::string tool;
@@ -336,6 +326,11 @@ int createStaticLibrary() {
     args.push_back(objfile);
   }
 
+  // user libs
+  for (auto libfile : global.params.libfiles) {
+    args.push_back(libfile);
+  }
+
   // .res/.def files for lib.exe
   if (isTargetMSVC) {
     if (global.params.resfile)
@@ -347,7 +342,6 @@ int createStaticLibrary() {
   // create path to the library
   createDirectoryForFileOrFail(libName);
 
-#if LDC_LLVM_VER >= 309
   if (useInternalArchiver) {
     const auto fullArgs =
         getFullArgs(tool.c_str(), args, global.params.verbose);
@@ -359,7 +353,6 @@ int createStaticLibrary() {
 
     return exitCode;
   }
-#endif
 
   // invoke external archiver
   return executeToolAndWait(tool, args, global.params.verbose);
