@@ -305,13 +305,9 @@ void outputIR2ObjRelevantCmdlineArgs(llvm::raw_ostream &hash_os) {
   hash_os << opts::getCPUStr();
   hash_os << opts::getFeaturesStr();
   hash_os << opts::floatABI;
-#if LDC_LLVM_VER >= 309
   const auto relocModel = opts::getRelocModel();
   if (relocModel.hasValue())
     hash_os << relocModel.getValue();
-#else
-  hash_os << opts::getRelocModel();
-#endif
 #if LDC_LLVM_VER >= 600
   const auto codeModel = opts::getCodeModel();
   if (codeModel.hasValue())
@@ -319,7 +315,13 @@ void outputIR2ObjRelevantCmdlineArgs(llvm::raw_ostream &hash_os) {
 #else
   hash_os << opts::getCodeModel();
 #endif
+#if LDC_LLVM_VER >= 800
+  const auto framePointerUsage = opts::framePointerUsage();
+  if (framePointerUsage.hasValue())
+    hash_os << framePointerUsage.getValue();
+#else
   hash_os << opts::disableFPElim();
+#endif
 }
 
 // Output to `hash_os` all environment flags that influence object code output
@@ -336,7 +338,7 @@ void calculateModuleHash(llvm::Module *m, llvm::SmallString<32> &str) {
   raw_hash_ostream hash_os;
 
   // Let hash depend on the compiler version:
-  hash_os << global.ldc_version << global.version << global.llvm_version
+  hash_os << global.ldc_version << global.version.ptr << global.llvm_version
           << ldc::built_with_Dcompiler_version;
 
   // Let hash depend on compile flags that change the outputted obj file,
@@ -481,7 +483,13 @@ void recoverObjectFile(llvm::StringRef cacheObjectHash,
       fatal();
     }
 
-    if (llvm::sys::fs::setLastModificationAndAccessTime(FD, getTimeNow())) {
+#if LDC_LLVM_VER < 800
+#define SET_LAST_ACCESS_AND_MOD_TIME setLastModificationAndAccessTime
+#else
+#define SET_LAST_ACCESS_AND_MOD_TIME setLastAccessAndModificationTime
+#endif
+
+    if (llvm::sys::fs::SET_LAST_ACCESS_AND_MOD_TIME(FD, getTimeNow())) {
       error(Loc(), "Failed to set the cached file modification time: %s",
             cacheFile.c_str());
       fatal();
