@@ -8,8 +8,9 @@
 //===----------------------------------------------------------------------===//
 
 #include "driver/configfile.h"
+
+#include "dmd/mars.h"
 #include "driver/exe_path.h"
-#include "mars.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/Path.h"
@@ -18,6 +19,7 @@
 #include <cstring>
 #include <iostream>
 #include <string>
+
 #if _WIN32
 #define WIN32_LEAN_AND_MEAN
 #include "llvm/Support/ConvertUTF.h"
@@ -36,8 +38,6 @@ static llvm::cl::opt<std::string>
     clConf("conf", llvm::cl::desc("Use configuration file <filename>"),
            llvm::cl::value_desc("filename"), llvm::cl::ZeroOrMore);
 
-ConfigFile ConfigFile::instance;
-
 #if _WIN32
 std::string getUserHomeDirectory() {
   char buff[MAX_PATH];
@@ -65,7 +65,9 @@ static bool ReadPathFromRegistry(llvm::SmallString<128> &p) {
     DWORD length;
     if (RegGetValue(hkey, NULL, _T("Path"), RRF_RT_REG_SZ, NULL, NULL,
                     &length) == ERROR_SUCCESS) {
-      TCHAR *data = static_cast<TCHAR *>(_alloca(length * sizeof(TCHAR)));
+      std::vector<TCHAR> buffer;
+      buffer.reserve(length);
+      const auto data = buffer.data();
       if (RegGetValue(hkey, NULL, _T("Path"), RRF_RT_REG_SZ, NULL, data,
                       &length) == ERROR_SUCCESS) {
 #if UNICODE
