@@ -7,7 +7,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "errors.h"
+#include "dmd/errors.h"
 #include "driver/cl_options.h"
 #include "driver/cl_options_instrumentation.h"
 #include "driver/cl_options_sanitizers.h"
@@ -61,17 +61,17 @@ private:
   virtual void addCppStdlibLinkFlags(const llvm::Triple &triple);
   virtual void addProfileRuntimeLinkFlags(const llvm::Triple &triple);
   virtual void addXRayLinkFlags(const llvm::Triple &triple);
+  virtual bool addCompilerRTArchiveLinkFlags(llvm::StringRef baseName,
+                                             const llvm::Triple &triple);
 
   virtual void addLinker();
   virtual void addUserSwitches();
   void addDefaultPlatformLibs();
   virtual void addTargetFlags();
 
-#if LDC_LLVM_VER >= 309
   void addLTOGoldPluginFlags();
   void addDarwinLTOFlags();
   void addLTOLinkFlags();
-#endif
 
   virtual void addLdFlag(const llvm::Twine &flag) {
     args.push_back(("-Wl," + flag).str());
@@ -85,8 +85,6 @@ private:
 //////////////////////////////////////////////////////////////////////////////
 // LTO functionality
 
-#if LDC_LLVM_VER >= 309
-
 std::string getLTOGoldPluginPath() {
   if (!ltoLibrary.empty()) {
     if (llvm::sys::fs::exists(ltoLibrary))
@@ -188,8 +186,6 @@ void ArgsBuilder::addLTOLinkFlags() {
   }
 }
 
-#endif // LDC_LLVM_VER >= 309
-
 //////////////////////////////////////////////////////////////////////////////
 
 // Returns the arch name as used in the compiler_rt libs.
@@ -342,23 +338,49 @@ void ArgsBuilder::addFuzzLinkFlags(const llvm::Triple &triple) {
 // Adds all required link flags for -fxray-instrument when the xray library is
 // found.
 void ArgsBuilder::addXRayLinkFlags(const llvm::Triple &triple) {
-  const auto searchPaths = getFullCompilerRTLibPathCandidates("xray", triple);
-
-  bool linkerDarwin = triple.isOSDarwin();
   if (!triple.isOSLinux())
-    warning(Loc(), "XRay is not (fully) supported on non-Linux target OS.");
+    warning(Loc(), "XRay may not be fully supported on non-Linux target OS.");
 
-  for (const auto &filepath : searchPaths) {
-    if (!linkerDarwin)
-      addLdFlag("--whole-archive");
-
-    args.push_back(filepath);
-
-    if (!linkerDarwin)
-      addLdFlag("--no-whole-archive");
-
-    return;
+  bool libraryFoundAndLinked = addCompilerRTArchiveLinkFlags("xray", triple);
+#if LDC_LLVM_VER >= 700
+  // Since LLVM 7, each XRay mode was split into its own library.
+  if (libraryFoundAndLinked) {
+    addCompilerRTArchiveLinkFlags("xray-basic", triple);
+    addCompilerRTArchiveLinkFlags("xray-fdr", triple);
   }
+#else
+  // Before LLVM 7, XRay requires the C++ std library (but not on Darwin).
+  // Only link with the C++ stdlib when the XRay library was found.
+  if (libraryFoundAndLinked && !triple.isOSDarwin())
+    addCppStdlibLinkFlags(triple);
+#endif
+}
+
+// Returns true if library was found and added to link flags.
+bool ArgsBuilder::addCompilerRTArchiveLinkFlags(llvm::StringRef baseName,
+                                                const llvm::Triple &triple) {
+  const bool linkerDarwin = triple.isOSDarwin();
+  const auto searchPaths = getFullCompilerRTLibPathCandidates(baseName, triple);
+  for (const auto &filepath : searchPaths) {
+    IF_LOG Logger::println("Searching runtime library: %s", filepath.c_str());
+
+    if (llvm::sys::fs::exists(filepath) &&
+        !llvm::sys::fs::is_directory(filepath)) {
+      if (!linkerDarwin)
+        addLdFlag("--whole-archive");
+
+      IF_LOG Logger::println("Found, linking with %s",
+                             filepath.c_str());
+      args.push_back(filepath);
+
+      if (!linkerDarwin)
+        addLdFlag("--no-whole-archive");
+
+      return true;
+    }
+  }
+
+  return false;
 }
 
 void ArgsBuilder::addCppStdlibLinkFlags(const llvm::Triple &triple) {
@@ -395,13 +417,11 @@ void ArgsBuilder::addProfileRuntimeLinkFlags(const llvm::Triple &triple) {
   const auto searchPaths =
       getFullCompilerRTLibPathCandidates("profile", triple);
 
-#if LDC_LLVM_VER >= 308
   if (global.params.targetTriple->isOSLinux()) {
     // For Linux, explicitly define __llvm_profile_runtime as undefined
     // symbol, so that the initialization part of profile-rt is linked in.
     addLdFlag("-u", llvm::getInstrProfRuntimeHookVarName());
   }
-#endif
 
   for (const auto &filepath : searchPaths) {
     IF_LOG Logger::println("Searching profile runtime: %s", filepath.c_str());
@@ -480,12 +500,10 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
     addXRayLinkFlags(*global.params.targetTriple);
   }
 
-#if LDC_LLVM_VER >= 309
   // Add LTO link flags before adding the user link switches, such that the user
   // can pass additional options to the LTO plugin.
   if (opts::isUsingLTO())
     addLTOLinkFlags();
-#endif
 
   addLinker();
   addUserSwitches();
@@ -534,8 +552,12 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
 void ArgsBuilder::addLinker() {
   if (!opts::linker.empty()) {
     args.push_back("-fuse-ld=" + opts::linker);
-  } else if (global.params.isLinux && opts::isUsingThinLTO()) {
-    // default to ld.gold for ThinLTO on Linux due to ld.bfd issues (see #2278)
+  } else if (global.params.isLinux &&
+             global.params.targetTriple->getEnvironment() !=
+                 llvm::Triple::Android) {
+    // Default to ld.gold on Linux due to ld.bfd issues with ThinLTO (see #2278)
+    // and older bfd versions stripping llvm.used symbols (e.g., ModuleInfo
+    // refs) with --gc-sections (see #2870).
     args.push_back("-fuse-ld=gold");
   }
 }
@@ -709,16 +731,28 @@ int linkObjToBinaryGcc(llvm::StringRef outputPath,
   }
 #endif
 
-  // find gcc for linking
-  const std::string tool = getGcc();
+  // build command-line for gcc-compatible linker driver
+  // exception: invoke (ld-compatible) linker directly for WebAssembly targets
+  std::string tool;
+  std::unique_ptr<ArgsBuilder> argsBuilder;
+#if LDC_LLVM_VER >= 500
+  if (global.params.targetTriple->isOSBinFormatWasm()) {
+    tool = getProgram("wasm-ld", &opts::linker);
+    argsBuilder = llvm::make_unique<LdArgsBuilder>();
+  } else {
+#endif
+    tool = getGcc();
+    argsBuilder = llvm::make_unique<ArgsBuilder>();
+#if LDC_LLVM_VER >= 500
+  }
+#endif
 
   // build arguments
-  ArgsBuilder argsBuilder;
-  argsBuilder.build(outputPath, defaultLibNames);
+  argsBuilder->build(outputPath, defaultLibNames);
 
   Logger::println("Linking with: ");
   Stream logstr = Logger::cout();
-  for (const auto &arg : argsBuilder.args) {
+  for (const auto &arg : argsBuilder->args) {
     if (!arg.empty()) {
       logstr << "'" << arg << "' ";
     }
@@ -726,5 +760,5 @@ int linkObjToBinaryGcc(llvm::StringRef outputPath,
   logstr << "\n"; // FIXME where's flush ?
 
   // try to call linker
-  return executeToolAndWait(tool, argsBuilder.args, global.params.verbose);
+  return executeToolAndWait(tool, argsBuilder->args, global.params.verbose);
 }
