@@ -7,17 +7,19 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "module.h"
-#include "errors.h"
-#include "id.h"
-#include "hdrgen.h"
-#include "json.h"
-#include "mars.h"
-#include "mtype.h"
-#include "identifier.h"
-#include "rmem.h"
-#include "root.h"
-#include "scope.h"
+#include "dmd/compiler.h"
+#include "dmd/cond.h"
+#include "dmd/errors.h"
+#include "dmd/id.h"
+#include "dmd/identifier.h"
+#include "dmd/hdrgen.h"
+#include "dmd/json.h"
+#include "dmd/mars.h"
+#include "dmd/module.h"
+#include "dmd/mtype.h"
+#include "dmd/root/rmem.h"
+#include "dmd/root/root.h"
+#include "dmd/scope.h"
 #include "dmd/target.h"
 #include "driver/cache.h"
 #include "driver/cl_options.h"
@@ -31,6 +33,7 @@
 #include "driver/linker.h"
 #include "driver/plugins.h"
 #include "driver/targetmachine.h"
+#include "gen/abi.h"
 #include "gen/cl_helpers.h"
 #include "gen/irstate.h"
 #include "gen/ldctraits.h"
@@ -45,48 +48,37 @@
 #include "gen/passes/Passes.h"
 #include "gen/runtime.h"
 #include "gen/uda.h"
-#include "gen/abi.h"
 #include "llvm/InitializePasses.h"
+#include "llvm/IR/LLVMContext.h"
+#include "llvm/LinkAllIR.h"
 #include "llvm/LinkAllPasses.h"
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/Host.h"
 #include "llvm/Support/ManagedStatic.h"
 #include "llvm/Support/Path.h"
-#if LDC_LLVM_VER >= 308
 #include "llvm/Support/StringSaver.h"
-#endif
 #include "llvm/Support/TargetRegistry.h"
 #include "llvm/Support/TargetSelect.h"
 #include "llvm/Target/TargetMachine.h"
+#include <assert.h>
+#include <limits.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #if LDC_LLVM_VER >= 600
 #include "llvm/CodeGen/TargetSubtargetInfo.h"
 #else
 #include "llvm/Target/TargetSubtargetInfo.h"
 #endif
-#include "llvm/LinkAllIR.h"
-#include "llvm/IR/LLVMContext.h"
-#include <assert.h>
-#include <limits.h>
-#include <stdio.h>
-#include <stdlib.h>
+
 #if _WIN32
 #include <windows.h>
 #endif
 
-// Needs Type already declared.
-#include "cond.h"
-
-// From druntime/src/core/runtime.d.
-extern "C" {
-int rt_init();
-}
-
 // In dmd/doc.d
 void gendocfile(Module *m);
 
 // In dmd/mars.d
-extern bool includeImports;
-extern Strings includeModulePatterns;
 void generateJson(Modules *modules);
 
 using namespace opts;
@@ -99,10 +91,16 @@ static cl::list<std::string, StringsAdapter>
                 cl::value_desc("directory"), cl::location(impPathsStore),
                 cl::Prefix);
 
+// Note: this option is parsed manually in C main().
+static cl::opt<bool> enableGC(
+    "lowmem", cl::ZeroOrMore,
+    cl::desc("Enable the garbage collector for the LDC front-end. This reduces "
+             "the compiler memory requirements but increases compile times."));
+
 // This function exits the program.
 void printVersion(llvm::raw_ostream &OS) {
   OS << "LDC - the LLVM D compiler (" << global.ldc_version << "):\n";
-  OS << "  based on DMD " << global.version << " and LLVM "
+  OS << "  based on DMD " << global.version.ptr << " and LLVM "
      << global.llvm_version << "\n";
   OS << "  built with " << ldc::built_with_Dcompiler_version << "\n";
 #if defined(__has_feature)
@@ -143,9 +141,6 @@ void printVersionStdout() {
 
 namespace {
 
-// True when target triple has an uClibc environment
-bool isUClibc = false;
-
 // Helper function to handle -d-debug=* and -d-version=*
 void processVersions(std::vector<std::string> &list, const char *type,
                      unsigned &globalLevel, Strings *&globalIDs) {
@@ -174,48 +169,6 @@ void processVersions(std::vector<std::string> &list, const char *type,
   }
 }
 
-// Helper function to handle -transition=*
-void processTransitions(std::vector<std::string> &list) {
-  for (const auto &i : list) {
-    if (i == "?") {
-      printf("\n"
-             "Language changes listed by -transition=id:\n"
-             "  =all           list information on all language changes\n"
-             "  =checkimports  give deprecation messages about 10378 "
-             "anomalies\n"
-             "  =complex,14488 list all usages of complex or imaginary types\n"
-             "  =field,3449    list all non-mutable fields which occupy an "
-             "object instance\n"
-             "  =import,10378  revert to single phase name lookup\n"
-             "  =intpromote,16997 fix integral promotions for unary + - ~ "
-             "operators\n"
-             "  =tls           list all variables going into thread local "
-             "storage\n");
-      exit(EXIT_SUCCESS);
-    } else if (i == "all") {
-      global.params.vtls = true;
-      global.params.vfield = true;
-      global.params.vcomplex = true;
-      global.params.bug10378 = true;   // not set in DMD
-      global.params.check10378 = true; // not set in DMD
-    } else if (i == "checkimports") {
-      global.params.check10378 = true;
-    } else if (i == "complex" || i == "14488") {
-      global.params.vcomplex = true;
-    } else if (i == "field" || i == "3449") {
-      global.params.vfield = true;
-    } else if (i == "import" || i == "10378") {
-      global.params.bug10378 = true;
-    } else if (i == "intpromote" || i == "16997") {
-      global.params.fix16997 = true;
-    } else if (i == "tls") {
-      global.params.vtls = true;
-    } else {
-      error(Loc(), "Invalid transition %s", i.c_str());
-    }
-  }
-}
-
 template <int N> // option length incl. terminating null
 void tryParse(const llvm::SmallVectorImpl<const char *> &args, size_t i,
               const char *&output, const char (&option)[N]) {
@@ -272,7 +225,6 @@ tryGetExplicitTriple(const llvm::SmallVectorImpl<const char *> &args) {
 
 void expandResponseFiles(llvm::BumpPtrAllocator &A,
                          llvm::SmallVectorImpl<const char *> &args) {
-#if LDC_LLVM_VER >= 308
   llvm::StringSaver Saver(A);
   cl::ExpandResponseFiles(Saver,
 #ifdef _WIN32
@@ -282,16 +234,15 @@ void expandResponseFiles(llvm::BumpPtrAllocator &A,
 #endif
                           ,
                           args);
-#endif
 }
 
 /// Parses switches from the command line, any response files and the global
 /// config file and sets up global.params accordingly.
 ///
 /// Returns a list of source file names.
-void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
-                      bool &helpOnly) {
-  global.params.argv0 = exe_path::getExePath().data();
+void parseCommandLine(int argc, char **argv, Strings &sourceFiles) {
+  const auto &exePath = exe_path::getExePath();
+  global.params.argv0 = {exePath.length(), exePath.data()};
 
   // Set up `opts::allArguments`, the combined list of command line arguments.
   using opts::allArguments;
@@ -328,8 +279,7 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
                               const_cast<char **>(allArguments.data()),
                               "LDC - the LLVM D compiler\n");
 
-  helpOnly = opts::printTargetFeaturesHelp();
-  if (helpOnly) {
+  if (opts::printTargetFeaturesHelp()) {
     auto triple = llvm::Triple(cfg_triple);
     std::string errMsg;
     if (auto target = lookupTarget("", triple, errMsg)) {
@@ -340,7 +290,7 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
       error(Loc(), "%s", errMsg.c_str());
       fatal();
     }
-    return;
+    exit(EXIT_SUCCESS);
   }
 
   if (!cfg_file.path().empty())
@@ -353,7 +303,7 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
   if (global.params.verbose) {
     message("binary    %s", exe_path::getExePath().c_str());
     message("version   %s (DMD %s, LLVM %s)", global.ldc_version,
-            global.version, global.llvm_version);
+            global.version.ptr, global.llvm_version);
     if (global.inifilename) {
       message("config    %s (%s)", global.inifilename, cfg_triple.c_str());
     }
@@ -362,7 +312,6 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
   // Negated options
   global.params.link = !compileOnly;
   global.params.obj = !dontWriteObj;
-  global.params.release = !opts::invReleaseMode;
   global.params.useInlineAsm = !noAsm;
 
   // String options: std::string --> char*
@@ -383,6 +332,8 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
   global.params.doHdrGeneration |=
       global.params.hdrdir || global.params.hdrname;
 
+  opts::initFromPathString(global.params.mixinFile, mixinFile);
+
   if (moduleDeps.getNumOccurrences() != 0) {
     global.params.moduleDeps = new OutBuffer;
     if (!moduleDeps.empty())
@@ -433,12 +384,18 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
   processVersions(versions, "version", global.params.versionlevel,
                   global.params.versionids);
 
-  processTransitions(transitions);
+  for (const auto &id : transitions)
+    parseTransitionOption(global.params, id.c_str());
+  for (const auto &id : previews)
+    parsePreviewOption(global.params, id.c_str());
+  for (const auto &id : reverts)
+    parseRevertOption(global.params, id.c_str());
 
-  if (useDIP1000) {
+  // -preview=dip1000 implies -preview=dip25 too
+  if (global.params.vsafe)
     global.params.useDIP25 = true;
-    global.params.vsafe = true;
-  }
+  if (global.params.noDIP25)
+    global.params.useDIP25 = false;
 
   global.params.output_o =
       (opts::output_o == cl::BOU_UNSET &&
@@ -494,28 +451,6 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
     }
   }
 
-  if (global.params.betterC) {
-    global.params.checkAction = CHECKACTION_C;
-    global.params.useModuleInfo = false;
-    global.params.useTypeInfo = false;
-    global.params.useExceptions = false;
-  }
-
-  if (global.params.useUnitTests) {
-    global.params.useAssert = CHECKENABLEon;
-  }
-
-  // -release downgrades default checks
-  if (global.params.useArrayBounds == CHECKENABLEdefault)
-    global.params.useArrayBounds =
-        global.params.release ? CHECKENABLEsafeonly : CHECKENABLEon;
-  if (global.params.useAssert == CHECKENABLEdefault)
-    global.params.useAssert =
-        global.params.release ? CHECKENABLEoff : CHECKENABLEon;
-  if (global.params.useSwitchError == CHECKENABLEdefault)
-    global.params.useSwitchError =
-        global.params.release ? CHECKENABLEoff : CHECKENABLEon;
-
   // LDC output determination
 
   // if we don't link and there's no `-output-*` switch but an `-of` one,
@@ -540,15 +475,6 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
     }
   }
 
-  // only link if possible
-  if (!global.params.obj || !global.params.output_o || global.params.lib) {
-    global.params.link = false;
-  }
-
-  if (global.params.lib && global.params.dll) {
-    error(Loc(), "-lib and -shared switches cannot be used together");
-  }
-
   if (soname.getNumOccurrences() > 0 && !global.params.dll) {
     error(Loc(), "-soname can be used only when building a shared library");
   }
@@ -571,15 +497,10 @@ void initializePasses() {
   initializeInstrumentation(Registry);
   initializeAnalysis(Registry);
   initializeCodeGen(Registry);
-#if LDC_LLVM_VER >= 309
   initializeGlobalISel(Registry);
-#endif
   initializeTarget(Registry);
 
 // Initialize passes not included above
-#if LDC_LLVM_VER < 308
-  initializeIPA(Registry);
-#endif
 #if LDC_LLVM_VER >= 400
   initializeRewriteSymbolsLegacyPassPass(Registry);
 #else
@@ -617,7 +538,7 @@ void fixupUClibcEnv() {
   envName.replace(0, 6, "gnu");
   triple.setEnvironmentName(envName);
   mTargetTriple = triple.normalize();
-  isUClibc = true;
+  global.params.isUClibcEnvironment = true;
 }
 
 /// Register the float ABI.
@@ -792,6 +713,7 @@ void registerPredefinedTargetVersions() {
                                                                      : "Win32");
     if (triple.isWindowsMSVCEnvironment()) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Microsoft");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Microsoft");
     }
     if (triple.isWindowsGNUEnvironment()) {
       VersionCondition::addPredefinedGlobalIdent(
@@ -810,12 +732,13 @@ void registerPredefinedTargetVersions() {
     if (triple.getEnvironment() == llvm::Triple::Android) {
       VersionCondition::addPredefinedGlobalIdent("Android");
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Bionic");
-    } else if (isMusl()) {
+    } else if (triple.isMusl()) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Musl");
-    } else if (isUClibc) {
+    } else if (global.params.isUClibcEnvironment) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_UClibc");
     } else {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Glibc");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
     }
     break;
   case llvm::Triple::Haiku:
@@ -828,18 +751,22 @@ void registerPredefinedTargetVersions() {
     VersionCondition::addPredefinedGlobalIdent(
         "darwin"); // For backwards compatibility.
     VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
     break;
   case llvm::Triple::FreeBSD:
     VersionCondition::addPredefinedGlobalIdent("FreeBSD");
     VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
     break;
   case llvm::Triple::Solaris:
     VersionCondition::addPredefinedGlobalIdent("Solaris");
     VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Sun");
     break;
   case llvm::Triple::DragonFly:
     VersionCondition::addPredefinedGlobalIdent("DragonFlyBSD");
     VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
     break;
   case llvm::Triple::NetBSD:
     VersionCondition::addPredefinedGlobalIdent("NetBSD");
@@ -848,6 +775,7 @@ void registerPredefinedTargetVersions() {
   case llvm::Triple::OpenBSD:
     VersionCondition::addPredefinedGlobalIdent("OpenBSD");
     VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
     break;
   case llvm::Triple::AIX:
     VersionCondition::addPredefinedGlobalIdent("AIX");
@@ -863,6 +791,8 @@ void registerPredefinedTargetVersions() {
   }
 }
 
+} // anonymous namespace
+
 /// Registers all predefined D version identifiers for the current
 /// configuration with VersionCondition.
 void registerPredefinedVersions() {
@@ -934,19 +864,65 @@ void registerPredefinedVersions() {
 #undef STR
 }
 
-} // anonymous namespace
+extern "C" {
+// in driver/main.d:
+int _Dmain(/*string[] args*/);
+
+// in druntime:
+int _d_run_main(int argc, char **argv, int (*dMain)());
+void gc_disable();
+}
+
+/// LDC's entry point, C main.
+/// Without `-lowmem`, we need to switch to the bump-pointer allocation scheme
+/// right from the start, before any module ctors are run, so we need this hook
+/// before druntime is initialized and `_Dmain` is called.
+int main(int argc, char **argv) {
+  bool lowmem = false;
+  for (int i = 1; i < argc; ++i) {
+    if (strncmp(argv[i], "-lowmem", 7) == 0) {
+      auto remainder = argv[i] + 7;
+      if (remainder[0] == 0) {
+        lowmem = true;
+      } else if (remainder[0] == '=') {
+        lowmem = strcmp(remainder + 1, "true") == 0 ||
+                 strcmp(remainder + 1, "TRUE") == 0;
+      }
+    }
+  }
+
+  if (!lowmem)
+    mem.disableGC();
+
+  // Initialize druntime; _Dmain() just calls cppmain() below with the original
+  // C args.
+  return _d_run_main(argc, argv, _Dmain);
+}
 
 int cppmain(int argc, char **argv) {
-#if LDC_LLVM_VER >= 309
   llvm::sys::PrintStackTraceOnErrorSignal(argv[0]);
-#else
-  llvm::sys::PrintStackTraceOnErrorSignal();
-#endif
+
+  // Older host druntime versions need druntime to be initialized before
+  // disabling the GC, so we cannot disable it in C main above.
+  if (!mem.isGCEnabled())
+    gc_disable();
 
   exe_path::initialize(argv[0]);
 
+  // Filter out druntime options in the cmdline, e.g., to configure the GC.
+  std::vector<char *> filteredArgs;
+  filteredArgs.reserve(argc);
+  for (int i = 0; i < argc; ++i) {
+    if (strncmp(argv[i], "--DRT-", 6) != 0)
+      filteredArgs.push_back(argv[i]);
+  }
+
+  argc = static_cast<int>(filteredArgs.size());
+  argv = filteredArgs.data();
+
   global._init();
-  global.version = ldc::dmd_version;
+  // global.version includes the terminating null
+  global.version = {strlen(ldc::dmd_version) + 1, ldc::dmd_version};
   global.ldc_version = ldc::ldc_version;
   global.llvm_version = ldc::llvm_version;
 
@@ -960,22 +936,12 @@ int cppmain(int argc, char **argv) {
 
   initializePasses();
 
-  bool helpOnly;
   Strings files;
-  parseCommandLine(argc, argv, files, helpOnly);
-
-  if (helpOnly) {
-    return 0;
-  }
-
-  if (files.dim == 0) {
-    if (global.params.jsonFieldFlags) {
-      generateJson(nullptr);
-      return EXIT_SUCCESS;
-    }
+  parseCommandLine(argc, argv, files);
 
+  if (argc == 1) {
     cl::PrintHelpMessage(/*Hidden=*/false, /*Categorized=*/true);
-    return EXIT_FAILURE;
+    exit(EXIT_FAILURE);
   }
 
   if (global.errors) {
@@ -1000,11 +966,7 @@ int cppmain(int argc, char **argv) {
   }
 
   auto relocModel = getRelocModel();
-#if LDC_LLVM_VER >= 309
   if (global.params.dll && !relocModel.hasValue()) {
-#else
-  if (global.params.dll && relocModel == llvm::Reloc::Default) {
-#endif
     relocModel = llvm::Reloc::PIC_;
   }
 
@@ -1019,12 +981,8 @@ int cppmain(int argc, char **argv) {
 
   opts::setDefaultMathOptions(gTargetMachine->Options);
 
-#if LDC_LLVM_VER >= 308
   static llvm::DataLayout DL = gTargetMachine->createDataLayout();
   gDataLayout = &DL;
-#else
-  gDataLayout = gTargetMachine->getDataLayout();
-#endif
 
   {
     llvm::Triple *triple = new llvm::Triple(gTargetMachine->getTargetTriple());
@@ -1064,11 +1022,9 @@ int cppmain(int argc, char **argv) {
   loadAllPlugins();
 
   Strings libmodules;
-  return mars_mainBody(files, libmodules);
+  return mars_mainBody(global.params, files, libmodules);
 }
 
-void addDefaultVersionIdentifiers() { registerPredefinedVersions(); }
-
 void codegenModules(Modules &modules) {
   // Generate one or more object/IR/bitcode files/dcompute kernels.
   if (global.params.obj && !modules.empty()) {
@@ -1085,6 +1041,10 @@ void codegenModules(Modules &modules) {
     // codegenned.
     for (d_size_t i = modules.dim; i-- > 0;) {
       Module *const m = modules[i];
+
+      if (m->isHdrFile)
+        continue;
+
       if (global.params.verbose)
         message("code      %s", m->toChars());
 
@@ -1097,7 +1057,7 @@ void codegenModules(Modules &modules) {
         computeModules.push_back(m);
         if (atCompute == DComputeCompileFor::deviceOnly) {
           // Remove m's object file from list of object files
-          auto s = m->objfile->name->str;
+          auto s = m->objfile->name.toChars();
           for (size_t j = 0; j < global.params.objfiles.dim; j++) {
             if (s == global.params.objfiles[j]) {
               global.params.objfiles.remove(j);
@@ -1113,9 +1073,9 @@ void codegenModules(Modules &modules) {
     if (!computeModules.empty()) {
       for (auto &mod : computeModules)
         dccg.emit(mod);
-
-      dccg.writeModules();
     }
+    dccg.writeModules();
+
     // We may have removed all object files, if so don't link.
     if (global.params.objfiles.dim == 0)
       global.params.link = false;
