@@ -18,9 +18,7 @@
 #include "gen/optimizer.h"
 #include "llvm/IR/AssemblyAnnotationWriter.h"
 #include "llvm/IR/Verifier.h"
-#if LDC_LLVM_VER >= 309
 #include "llvm/Analysis/ModuleSummaryAnalysis.h"
-#endif
 #if LDC_LLVM_VER >= 400
 #include "llvm/Analysis/ProfileSummaryInfo.h"
 #include "llvm/Bitcode/BitcodeWriter.h"
@@ -33,9 +31,6 @@
 #include "llvm/Support/FormattedStream.h"
 #include "llvm/Support/Program.h"
 #include "llvm/Support/Path.h"
-#ifdef LDC_LLVM_SUPPORTED_TARGET_SPIRV
-#include "llvm/Support/SPIRV.h"
-#endif
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Analysis/TargetTransformInfo.h"
 #if LDC_LLVM_VER >= 600
@@ -48,6 +43,12 @@
 #include <cstddef>
 #include <fstream>
 
+#ifdef LDC_LLVM_SUPPORTED_TARGET_SPIRV
+namespace llvm {
+    ModulePass *createSPIRVWriterPass(llvm::raw_ostream &Str);
+}
+#endif
+
 static llvm::cl::opt<bool>
     NoIntegratedAssembler("no-integrated-as", llvm::cl::ZeroOrMore,
                           llvm::cl::Hidden,
@@ -295,9 +296,6 @@ bool shouldOutputObjectFile() {
 }
 
 bool shouldDoLTO(llvm::Module *m) {
-#if LDC_LLVM_VER < 309
-  return false;
-#else
 #if LDC_LLVM_VER == 309
   // LLVM 3.9 bug: can't do ThinLTO with modules that have module-scope inline
   // assembly blocks (duplicate definitions upon importing from such a module).
@@ -306,7 +304,6 @@ bool shouldDoLTO(llvm::Module *m) {
     return false;
 #endif
   return opts::isUsingLTO();
-#endif
 }
 } // end of anonymous namespace
 
@@ -387,7 +384,6 @@ void writeModule(llvm::Module *m, const char *filename) {
 #endif
 
     if (opts::isUsingThinLTO()) {
-#if LDC_LLVM_VER >= 309
       Logger::println("Creating module summary for ThinLTO");
 #if LDC_LLVM_VER == 309
       // When the function freq info callback is set to nullptr, LLVM will
@@ -406,7 +402,6 @@ void writeModule(llvm::Module *m, const char *filename) {
 
       llvm::WriteBitcodeToFile(M, bos, true, &moduleSummaryIndex,
                                /* generate ThinLTO hash */ true);
-#endif
     } else {
       llvm::WriteBitcodeToFile(M, bos);
     }
