@@ -12,21 +12,20 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "mtype.h"
-#include "declaration.h"
-#include "aggregate.h"
+#include "gen/abi-win64.h"
 
+#include "dmd/mtype.h"
+#include "dmd/declaration.h"
+#include "dmd/aggregate.h"
+#include "gen/abi.h"
+#include "gen/abi-generic.h"
+#include "gen/dvalue.h"
 #include "gen/irstate.h"
 #include "gen/llvm.h"
-#include "gen/tollvm.h"
-#include "gen/logger.h"
-#include "gen/dvalue.h"
 #include "gen/llvmhelpers.h"
-#include "gen/abi.h"
-#include "gen/abi-win64.h"
-#include "gen/abi-generic.h"
+#include "gen/logger.h"
+#include "gen/tollvm.h"
 #include "ir/irfunction.h"
-
 #include <cassert>
 #include <string>
 #include <utility>
@@ -70,7 +69,7 @@ private:
       if (isMSVCpp && t->ty == Tstruct) {
         StructDeclaration *sd = static_cast<TypeStruct *>(t)->sym;
         assert(sd);
-        if (sd->postblit)
+        if (sd->postblit || sd->hasCopyCtor)
           return true;
       }
     }
@@ -90,7 +89,7 @@ public:
     // => let LLVM pass vectors in registers instead of passing a ref to a
     // hidden copy (both cases handled by LLVM automatically for LL vectors
     // which we don't rewrite).
-    return l == LINKd && !(tf && tf->varargs == 1)
+    return l == LINKd && !(tf && tf->parameterList.varargs == VARARGvariadic)
                ? llvm::CallingConv::X86_VectorCall
                : llvm::CallingConv::C;
   }
