@@ -8,13 +8,14 @@
 //===----------------------------------------------------------------------===//
 
 #include "gen/arrays.h"
-#include "aggregate.h"
-#include "declaration.h"
-#include "dsymbol.h"
-#include "expression.h"
-#include "init.h"
-#include "module.h"
-#include "mtype.h"
+
+#include "dmd/aggregate.h"
+#include "dmd/declaration.h"
+#include "dmd/dsymbol.h"
+#include "dmd/expression.h"
+#include "dmd/init.h"
+#include "dmd/module.h"
+#include "dmd/mtype.h"
 #include "gen/dvalue.h"
 #include "gen/funcgenstate.h"
 #include "gen/irstate.h"
@@ -185,18 +186,27 @@ static Type *DtoArrayElementType(Type *arrayType) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-static void copySlice(Loc &loc, LLValue *dstarr, LLValue *sz1, LLValue *srcarr,
-                      LLValue *sz2, bool knownInBounds) {
+static LLValue *computeSize(LLValue *length, size_t elementSize) {
+  return elementSize == 1
+             ? length
+             : gIR->ir->CreateMul(length, DtoConstSize_t(elementSize));
+};
+
+static void copySlice(Loc &loc, LLValue *dstarr, LLValue *dstlen, LLValue *srcarr,
+                      LLValue *srclen, size_t elementSize, bool knownInBounds) {
   const bool checksEnabled =
       global.params.useAssert == CHECKENABLEon || gIR->emitArrayBoundsChecks();
   if (checksEnabled && !knownInBounds) {
     LLValue *fn = getRuntimeFunction(loc, gIR->module, "_d_array_slice_copy");
-    gIR->CreateCallOrInvoke(fn, dstarr, sz1, srcarr, sz2);
+    gIR->CreateCallOrInvoke(
+        fn, {dstarr, dstlen, srcarr, srclen, DtoConstSize_t(elementSize)}, "",
+        /*isNothrow=*/true);
   } else {
     // We might have dstarr == srcarr at compile time, but as long as
     // sz1 == 0 at runtime, this would probably still be legal (the C spec
     // is unclear here).
-    DtoMemCpy(dstarr, srcarr, sz1);
+    LLValue *size = computeSize(dstlen, elementSize);
+    DtoMemCpy(dstarr, srcarr, size);
   }
 }
 
@@ -244,12 +254,6 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
   LLValue *lhsPtr = DtoBitCast(realLhsPtr, getVoidPtrType());
   LLValue *lhsLength = DtoArrayLen(lhs);
 
-  auto computeSize = [](LLValue *length, size_t elementSize) {
-    return elementSize == 1
-               ? length
-               : gIR->ir->CreateMul(length, DtoConstSize_t(elementSize));
-  };
-
   // Be careful to handle void arrays correctly when modifying this (see tests
   // for DMD issue 7493).
   // TODO: This should use AssignExp::memset.
@@ -267,15 +271,24 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
     if (!needsDestruction && !needsPostblit) {
       // fast version
       const size_t elementSize = getTypeAllocSize(DtoMemType(elemType));
-      LLValue *lhsSize = computeSize(lhsLength, elementSize);
-
       if (rhs->isNull()) {
+        LLValue *lhsSize = computeSize(lhsLength, elementSize);
         DtoMemSetZero(lhsPtr, lhsSize);
       } else {
-        LLValue *rhsSize = computeSize(rhsLength, elementSize);
-        const bool knownInBounds =
+        bool knownInBounds =
             isConstructing || (t->ty == Tsarray && t2->ty == Tsarray);
-        copySlice(loc, lhsPtr, lhsSize, rhsPtr, rhsSize, knownInBounds);
+        if (!knownInBounds) {
+          if (auto constLhsLength = llvm::dyn_cast<LLConstantInt>(lhsLength)) {
+            if (auto constRhsLength =
+                    llvm::dyn_cast<LLConstantInt>(rhsLength)) {
+              if (constLhsLength->getValue() == constRhsLength->getValue()) {
+                knownInBounds = true;
+              }
+            }
+          }
+        }
+        copySlice(loc, lhsPtr, lhsLength, rhsPtr, rhsLength, elementSize,
+                  knownInBounds);
       }
     } else if (isConstructing) {
       LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_arrayctor");
@@ -620,11 +633,7 @@ void initializeArrayLiteral(IRState *p, ArrayLiteralExp *ale, LLValue *dstMem) {
       auto gvar = new llvm::GlobalVariable(gIR->module, constarr->getType(),
                                            true, LLGlobalValue::InternalLinkage,
                                            constarr, ".arrayliteral");
-#if LDC_LLVM_VER >= 309
       gvar->setUnnamedAddr(llvm::GlobalValue::UnnamedAddr::Global);
-#else
-      gvar->setUnnamedAddr(true);
-#endif
       DtoMemCpy(dstMem, gvar,
                 DtoConstSize_t(getTypeAllocSize(constarr->getType())));
     }
@@ -673,10 +682,13 @@ DSliceValue *DtoNewDynArray(Loc &loc, Type *arrayType, DValue *dim,
   IF_LOG Logger::println("DtoNewDynArray : %s", arrayType->toChars());
   LOG_SCOPE;
 
-  // get runtime function
   Type *eltType = arrayType->toBasetype()->nextOf();
-  bool zeroInit = eltType->isZeroInit();
 
+  if (eltType->size() == 0)
+    return DtoNullValue(arrayType, loc)->isSlice();
+
+  // get runtime function
+  bool zeroInit = eltType->isZeroInit();
   const char *fnname = defaultInit
                            ? (zeroInit ? "_d_newarrayT" : "_d_newarrayiT")
                            : "_d_newarrayU";
@@ -694,7 +706,10 @@ DSliceValue *DtoNewDynArray(Loc &loc, Type *arrayType, DValue *dim,
       gIR->CreateCallOrInvoke(fn, arrayTypeInfo, arrayLen, ".gc_mem")
           .getInstruction();
 
-  return getSlice(arrayType, newArray);
+  // return a DSliceValue with the well-known length for better optimizability
+  auto ptr =
+      DtoBitCast(DtoExtractValue(newArray, 1, ".ptr"), DtoPtrToType(eltType));
+  return new DSliceValue(arrayType, arrayLen, ptr);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -915,7 +930,7 @@ DSliceValue *DtoCatArrays(Loc &loc, Type *arrayType, Expression *exp1,
   }
 
   auto newArray =
-      gIR->funcGen().callOrInvoke(fn, args, ".appendedArray").getInstruction();
+      gIR->CreateCallOrInvoke(fn, args, ".appendedArray").getInstruction();
   return getSlice(arrayType, newArray);
 }
 
@@ -988,7 +1003,7 @@ LLValue *DtoArrayEqCmp_impl(Loc &loc, const char *func, DValue *l,
     args.push_back(DtoBitCast(tival, fn->getFunctionType()->getParamType(2)));
   }
 
-  return gIR->funcGen().callOrInvoke(fn, args).getInstruction();
+  return gIR->CreateCallOrInvoke(fn, args).getInstruction();
 }
 
 /// When `true` is returned, the type can be compared using `memcmp`.
@@ -1145,29 +1160,6 @@ LLValue *DtoArrayEquals(Loc &loc, TOK op, DValue *l, DValue *r) {
   return res;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-LLValue *DtoArrayCastLength(Loc &loc, LLValue *len, LLType *elemty,
-                            LLType *newelemty) {
-  IF_LOG Logger::println("DtoArrayCastLength");
-  LOG_SCOPE;
-
-  assert(len);
-  assert(elemty);
-  assert(newelemty);
-
-  size_t esz = getTypeAllocSize(elemty);
-  size_t nsz = getTypeAllocSize(newelemty);
-  if (esz == nsz) {
-    return len;
-  }
-
-  LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_array_cast_len");
-  return gIR
-      ->CreateCallOrInvoke(fn, len, LLConstantInt::get(DtoSize_t(), esz, false),
-                           LLConstantInt::get(DtoSize_t(), nsz, false))
-      .getInstruction();
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 LLValue *DtoDynArrayIs(TOK op, DValue *l, DValue *r) {
   assert(l);
@@ -1195,7 +1187,9 @@ LLValue *DtoArrayLen(DValue *v) {
     if (v->isLVal()) {
       return DtoLoad(DtoGEPi(DtoLVal(v), 0, 0), ".len");
     }
-    return gIR->ir->CreateExtractValue(DtoRVal(v), 0, ".len");
+    auto slice = v->isSlice();
+    assert(slice);
+    return slice->getLength();
   }
   if (t->ty == Tsarray) {
     assert(!v->isSlice());
@@ -1223,7 +1217,9 @@ LLValue *DtoArrayPtr(DValue *v) {
     } else if (v->isLVal()) {
       ptr = DtoLoad(DtoGEPi(DtoLVal(v), 0, 1), ".ptr");
     } else {
-      ptr = gIR->ir->CreateExtractValue(DtoRVal(v), 1, ".ptr");
+      auto slice = v->isSlice();
+      assert(slice);
+      ptr = slice->getPtr();
     }
   } else if (t->ty == Tsarray) {
     assert(!v->isSlice());
@@ -1267,31 +1263,37 @@ DValue *DtoCastArray(Loc &loc, DValue *u, Type *to) {
     LLValue *length = nullptr;
     LLValue *ptr = nullptr;
     if (fromtype->ty == Tsarray) {
-      uinteger_t len = static_cast<TypeSArray *>(fromtype)->dim->toUInteger();
-      length = DtoConstSize_t(len);
+      length = DtoConstSize_t(
+          static_cast<TypeSArray *>(fromtype)->dim->toUInteger());
       ptr = DtoLVal(u);
-      assert(isaPointer(ptr->getType()));
-      LLArrayType *arrty = isaArray(ptr->getType()->getContainedType(0));
-
-      if (arrty->getNumElements() * fromtype->nextOf()->size() %
-              totype->nextOf()->size() !=
-          0) {
-        error(loc,
-              "invalid cast from `%s` to `%s`, the element sizes don't line up",
-              fromtype->toChars(), totype->toChars());
-        fatal();
-      }
     } else {
       length = DtoArrayLen(u);
       ptr = DtoArrayPtr(u);
     }
 
-    LLType *ptrty = DtoArrayType(totype)->getContainedType(1);
-    LLType *ety = DtoMemType(fromtype->nextOf());
-
-    if (fromtype->nextOf()->size() != totype->nextOf()->size())
-      length = DtoArrayCastLength(loc, length, ety, ptrty->getContainedType(0));
+    const auto fsize = fromtype->nextOf()->size();
+    const auto tsize = totype->nextOf()->size();
+    if (fsize != tsize) {
+      if (auto constLength = isaConstantInt(length)) {
+        // compute new constant length: (constLength * fsize) / tsize
+        const auto totalSize = constLength->getZExtValue() * fsize;
+        if (totalSize % tsize != 0) {
+          error(loc,
+                "invalid cast from `%s` to `%s`, the element sizes don't "
+                "line up",
+                fromtype->toChars(), totype->toChars());
+          fatal();
+        }
+        length = DtoConstSize_t(totalSize / tsize);
+      } else if (fsize % tsize == 0) {
+        // compute new dynamic length: length * (fsize / tsize)
+        length = gIR->ir->CreateMul(length, DtoConstSize_t(fsize / tsize));
+      } else {
+        llvm_unreachable("should have been lowered to `__ArrayCast`");
+      }
+    }
 
+    LLType *ptrty = tolltype->getStructElementType(1);
     return new DSliceValue(to, length, DtoBitCast(ptr, ptrty));
   }
 
