@@ -8,7 +8,9 @@
 //===----------------------------------------------------------------------===//
 
 #include "gen/binops.h"
-#include "declaration.h"
+
+#include "dmd/declaration.h"
+#include "dmd/expression.h"
 #include "gen/complex.h"
 #include "gen/dvalue.h"
 #include "gen/irstate.h"
@@ -382,7 +384,7 @@ LLValue *DtoBinFloatsEquals(Loc &loc, DValue *lhs, DValue *rhs, TOK op) {
 
 LLValue *mergeVectorEquals(LLValue *resultsVector, TOK op) {
   // `resultsVector` is a vector of i1 values, the pair-wise results.
-  // Bitcast to an integer and checks the bits via additional integer
+  // Bitcast to an integer and check the bits via additional integer
   // comparison.
   const auto sizeInBits = getTypeBitSize(resultsVector->getType());
   LLType *integerType = LLType::getIntNTy(gIR->context(), sizeInBits);
