@@ -8,9 +8,10 @@
 //===----------------------------------------------------------------------===//
 
 #include "gen/cl_helpers.h"
-#include "mars.h"
-#include "rmem.h"
-#include "root.h"
+
+#include "dmd/mars.h"
+#include "dmd/root/rmem.h"
+#include "dmd/root/root.h"
 #include <algorithm>
 #include <cctype> // isupper, tolower
 #include <stdarg.h>
@@ -33,25 +34,19 @@ void initFromPathString(const char *&dest, const cl::opt<std::string> &src) {
   dest = nullptr;
   if (src.getNumOccurrences() != 0) {
     if (src.empty()) {
-      error(Loc(), "Expected argument to '-%s'",
-#if LDC_LLVM_VER >= 308
-            src.ArgStr.str().c_str()
-#else
-            src.ArgStr
-#endif
-      );
+      error(Loc(), "Expected argument to '-%s'", src.ArgStr.str().c_str());
     }
     dest = dupPathString(src);
   }
 }
 
-MultiSetter::MultiSetter(bool invert, bool *p, ...) {
+MultiSetter::MultiSetter(bool invert, CHECKENABLE *p, ...) {
   this->invert = invert;
   if (p) {
     locations.push_back(p);
     va_list va;
     va_start(va, p);
-    while ((p = va_arg(va, bool *))) {
+    while ((p = va_arg(va, CHECKENABLE *))) {
       locations.push_back(p);
     }
     va_end(va);
@@ -60,7 +55,7 @@ MultiSetter::MultiSetter(bool invert, bool *p, ...) {
 
 void MultiSetter::operator=(bool val) {
   for (auto &l : locations) {
-    *l = (val != invert);
+    *l = (val != invert) ? CHECKENABLEon : CHECKENABLEoff;
   }
 }
 
