@@ -7,17 +7,20 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "gen/llvm.h"
-#include "aggregate.h"
-#include "declaration.h"
-#include "init.h"
-#include "mtype.h"
-#include "target.h"
-#include "gen/arrays.h"
 #include "gen/classes.h"
+
+#include "dmd/aggregate.h"
+#include "dmd/declaration.h"
+#include "dmd/expression.h"
+#include "dmd/identifier.h"
+#include "dmd/init.h"
+#include "dmd/mtype.h"
+#include "dmd/target.h"
+#include "gen/arrays.h"
 #include "gen/dvalue.h"
 #include "gen/functions.h"
 #include "gen/irstate.h"
+#include "gen/llvm.h"
 #include "gen/llvmhelpers.h"
 #include "gen/logger.h"
 #include "gen/nested.h"
@@ -79,8 +82,12 @@ DValue *DtoNewClass(Loc &loc, TypeClass *tc, NewExp *newexp) {
 
   // allocate
   LLValue *mem;
+  bool doInit = true;
   if (newexp->onstack) {
-    mem = DtoRawAlloca(DtoType(tc)->getContainedType(0), DtoAlignment(tc),
+    unsigned alignment = tc->sym->alignsize;
+    if (alignment == STRUCTALIGN_DEFAULT)
+      alignment = 0;
+    mem = DtoRawAlloca(DtoType(tc)->getContainedType(0), alignment,
                        ".newclass_alloca");
   }
   // custom allocator
@@ -92,17 +99,23 @@ DValue *DtoNewClass(Loc &loc, TypeClass *tc, NewExp *newexp) {
   }
   // default allocator
   else {
-    llvm::Function *fn =
-        getRuntimeFunction(loc, gIR->module, "_d_allocclass");
+    const bool useEHAlloc = global.params.ehnogc && newexp->thrownew;
+    llvm::Function *fn = getRuntimeFunction(
+        loc, gIR->module, useEHAlloc ? "_d_newThrowable" : "_d_allocclass");
     LLConstant *ci = DtoBitCast(getIrAggr(tc->sym)->getClassInfoSymbol(),
                                 DtoType(getClassInfoType()));
-    mem =
-        gIR->CreateCallOrInvoke(fn, ci, ".newclass_gc_alloc").getInstruction();
-    mem = DtoBitCast(mem, DtoType(tc), ".newclass_gc");
+    mem = gIR->CreateCallOrInvoke(fn, ci,
+                                  useEHAlloc ? ".newthrowable_alloc"
+                                             : ".newclass_gc_alloc")
+              .getInstruction();
+    mem = DtoBitCast(mem, DtoType(tc),
+                     useEHAlloc ? ".newthrowable" : ".newclass_gc");
+    doInit = !useEHAlloc;
   }
 
   // init
-  DtoInitClass(tc, mem);
+  if (doInit)
+    DtoInitClass(tc, mem);
 
   // init inner-class outer reference
   if (newexp->thisexp) {
@@ -163,7 +176,7 @@ void DtoInitClass(TypeClass *tc, LLValue *dst) {
   // Copy the rest from the static initializer, if any.
   unsigned const firstDataIdx = isCPPclass ? 1 : 2;
   uint64_t const dataBytes =
-      tc->sym->structsize - Target::ptrsize * firstDataIdx;
+      tc->sym->structsize - target.ptrsize * firstDataIdx;
   if (dataBytes == 0) {
     return;
   }
@@ -191,26 +204,8 @@ void DtoFinalizeClass(Loc &loc, LLValue *inst) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void DtoFinalizeScopeClass(Loc &loc, LLValue *inst, ClassDeclaration *cd) {
-  if (!isOptimizationEnabled()) {
-    DtoFinalizeClass(loc, inst);
-    return;
-  }
-
-  assert(cd);
-  // As of 2.077, the front-end doesn't emit the implicit delete() for C++
-  // classes, so this code assumes D classes.
-  assert(!cd->isCPPclass());
-
-  bool hasDtor = false;
-  for (; cd; cd = cd->baseClass) {
-    if (cd->dtor) {
-      hasDtor = true;
-      break;
-    }
-  }
-
-  if (hasDtor) {
+void DtoFinalizeScopeClass(Loc &loc, LLValue *inst, bool hasDtor) {
+  if (!isOptimizationEnabled() || hasDtor) {
     DtoFinalizeClass(loc, inst);
     return;
   }
