@@ -7,14 +7,17 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "aggregate.h"
-#include "declaration.h"
-#include "enum.h"
-#include "id.h"
-#include "init.h"
-#include "nspace.h"
-#include "rmem.h"
-#include "template.h"
+#include "dmd/aggregate.h"
+#include "dmd/declaration.h"
+#include "dmd/enum.h"
+#include "dmd/expression.h"
+#include "dmd/id.h"
+#include "dmd/import.h"
+#include "dmd/init.h"
+#include "dmd/nspace.h"
+#include "dmd/root/rmem.h"
+#include "dmd/template.h"
+#include "driver/cl_options.h"
 #include "gen/classes.h"
 #include "gen/functions.h"
 #include "gen/irstate.h"
@@ -50,6 +53,15 @@ public:
 
   //////////////////////////////////////////////////////////////////////////
 
+  void visit(Import *im) override {
+    IF_LOG Logger::println("Import::codegen for %s", im->toPrettyChars());
+    LOG_SCOPE
+
+    irs->DBuilder.EmitImport(im);
+  }
+
+  //////////////////////////////////////////////////////////////////////////
+
   void visit(Nspace *ns) override {
     IF_LOG Logger::println("Nspace::codegen for %s", ns->toPrettyChars());
     LOG_SCOPE
@@ -74,7 +86,7 @@ public:
     }
 
     if (decl->type->ty == Terror) {
-      error(decl->loc, "had semantic errors when compiling");
+      decl->error("had semantic errors when compiling");
       decl->ir->setDefined();
       return;
     }
@@ -89,13 +101,10 @@ public:
       }
 
       // Emit TypeInfo.
-      if (global.params.useTypeInfo && Type::dtypeinfo) {
-        IrAggr *ir = getIrAggr(decl);
+      IrAggr *ir = getIrAggr(decl);
+      if (!ir->suppressTypeInfo() && !isSpeculativeType(decl->type)) {
         llvm::GlobalVariable *interfaceZ = ir->getClassInfoSymbol();
-        // Only define if not speculative.
-        if (!isSpeculativeType(decl->type)) {
-          defineGlobal(interfaceZ, ir->getClassInfoInit(), decl);
-        }
+        defineGlobal(interfaceZ, ir->getClassInfoInit(), decl);
       }
     }
   }
@@ -112,7 +121,7 @@ public:
     }
 
     if (decl->type->ty == Terror) {
-      error(decl->loc, "had semantic errors when compiling");
+      decl->error("had semantic errors when compiling");
       decl->ir->setDefined();
       return;
     }
@@ -139,20 +148,20 @@ public:
       setLinkage(decl, initGlobal);
 
       // emit typeinfo
-      if (global.params.useTypeInfo && Type::dtypeinfo) {
+      if (!ir->suppressTypeInfo()) {
         DtoTypeInfoOf(decl->type, /*base=*/false);
-      }
-    }
 
-    // Emit __xopEquals/__xopCmp/__xtoHash.
-    if (decl->xeq && decl->xeq != decl->xerreq) {
-      decl->xeq->accept(this);
-    }
-    if (decl->xcmp && decl->xcmp != decl->xerrcmp) {
-      decl->xcmp->accept(this);
-    }
-    if (decl->xhash) {
-      decl->xhash->accept(this);
+        // Emit __xopEquals/__xopCmp/__xtoHash.
+        if (decl->xeq && decl->xeq != decl->xerreq) {
+          decl->xeq->accept(this);
+        }
+        if (decl->xcmp && decl->xcmp != decl->xerrcmp) {
+          decl->xcmp->accept(this);
+        }
+        if (decl->xhash) {
+          decl->xhash->accept(this);
+        }
+      }
     }
   }
 
@@ -170,7 +179,7 @@ public:
     }
 
     if (decl->type->ty == Terror) {
-      error(decl->loc, "had semantic errors when compiling");
+      decl->error("had semantic errors when compiling");
       decl->ir->setDefined();
       return;
     }
@@ -196,12 +205,9 @@ public:
       ir->defineInterfaceVtbls();
 
       // Emit TypeInfo.
-      if (global.params.useTypeInfo && Type::dtypeinfo) {
+      if (!ir->suppressTypeInfo() && !isSpeculativeType(decl->type)) {
         llvm::GlobalVariable *classZ = ir->getClassInfoSymbol();
-        // Only define if not speculative.
-        if (!isSpeculativeType(decl->type)) {
-          defineGlobal(classZ, ir->getClassInfoInit(), decl);
-        }
+        defineGlobal(classZ, ir->getClassInfoInit(), decl);
       }
     }
   }
@@ -240,7 +246,7 @@ public:
     }
 
     if (decl->type->ty == Terror) {
-      error(decl->loc, "had semantic errors when compiling");
+      decl->error("had semantic errors when compiling");
       decl->ir->setDefined();
       return;
     }
@@ -297,6 +303,11 @@ public:
           gvar->setDLLStorageClass(LLGlobalValue::DLLExportStorageClass);
         }
 
+        // Hide non-exported symbols
+        if (opts::defaultToHiddenVisibility && !decl->isExport()) {
+          gvar->setVisibility(LLGlobalValue::HiddenVisibility);
+        }
+
         // Also set up the debug info.
         irs->DBuilder.EmitGlobalVariable(gvar, decl);
       }
@@ -319,7 +330,7 @@ public:
                            decl->toPrettyChars());
 
     if (decl->type->ty == Terror) {
-      error(decl->loc, "had semantic errors when compiling");
+      decl->error("had semantic errors when compiling");
       return;
     }
   }
@@ -369,6 +380,15 @@ public:
         Logger::println("Does not need codegen, skipping.");
         return;
       }
+
+      if (irs->dcomputetarget && (decl->tempdecl == Type::rtinfo ||
+                                  decl->tempdecl == Type::rtinfoImpl)) {
+        // Emitting object.RTInfo(Impl) template instantiations in dcompute
+        // modules would require dcompute support for global variables.
+        Logger::println("Skipping object.RTInfo(Impl) template instantiations "
+                        "in dcompute modules.");
+        return;
+      }
     }
 
     for (auto &m : *decl->members) {
@@ -409,16 +429,18 @@ public:
 
   //////////////////////////////////////////////////////////////////////////
 
+  static std::string getPragmaStringArg(PragmaDeclaration *decl) {
+    assert(decl->args && decl->args->dim == 1);
+    Expression *e = static_cast<Expression *>((*decl->args)[0]);
+    assert(e->op == TOKstring);
+    StringExp *se = static_cast<StringExp *>(e);
+    return std::string(se->toPtr(), se->numberOfCodeUnits());
+  }
+
   void visit(PragmaDeclaration *decl) override {
     if (decl->ident == Id::lib) {
-      assert(decl->args && decl->args->dim == 1);
       assert(!irs->dcomputetarget);
-        
-      Expression *e = static_cast<Expression *>(decl->args->data[0]);
-
-      assert(e->op == TOKstring);
-      StringExp *se = static_cast<StringExp *>(e);
-      const std::string name(se->toPtr(), se->numberOfCodeUnits());
+      const std::string name = getPragmaStringArg(decl);
       auto nameLen = name.size();
 
       if (global.params.targetTriple->isWindowsGNUEnvironment()) {
@@ -467,6 +489,14 @@ public:
         arg[n - 1] = 0;
         global.params.linkswitches.push(arg);
       }
+    } else if (decl->ident == Id::linkerDirective) {
+      if (global.params.targetTriple->isWindowsMSVCEnvironment()) {
+        // Embed directly as linker option in object file
+        const std::string directive = getPragmaStringArg(decl);
+        auto Value = llvm::MDString::get(gIR->context(), directive);
+        gIR->LinkerMetadataArgs.push_back(
+            llvm::MDNode::get(gIR->context(), Value));
+      }
     }
     visit(static_cast<AttribDeclaration *>(decl));
   }
