@@ -13,23 +13,19 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef LDC_GEN_LLVMHELPERS_H
-#define LDC_GEN_LLVMHELPERS_H
+#pragma once
 
+#include "dmd/mtype.h"
+#include "dmd/statement.h"
 #include "gen/dvalue.h"
 #include "gen/llvm.h"
 #include "ir/irfuncty.h"
-#include "mtype.h"
-#include "statement.h"
 
 struct IRState;
 
 // An arrayreference type with initializer_list support (C++11):
 template <class T> using ArrayParam = llvm::ArrayRef<T>;
 
-// Helper function because LLVM's isMusl wasn't around before 3.9.
-bool isMusl();
-
 llvm::LLVMContext& getGlobalContext();
 
 // dynamic memory helpers
@@ -298,5 +294,3 @@ DValue *makeVarDValue(Type *type, VarDeclaration *vd,
 bool toInPlaceConstruction(DLValue *lhs, Expression *rhs);
 
 std::string llvmTypeToString(LLType *type);
-
-#endif
