@@ -9,7 +9,7 @@
 
 #include "gen/optimizer.h"
 
-#include "errors.h"
+#include "dmd/errors.h"
 #include "gen/cl_helpers.h"
 #include "gen/logger.h"
 #include "gen/passes/Passes.h"
@@ -17,23 +17,28 @@
 #include "driver/cl_options_instrumentation.h"
 #include "driver/cl_options_sanitizers.h"
 #include "driver/targetmachine.h"
-#include "llvm/LinkAllPasses.h"
+#include "llvm/ADT/Triple.h"
+#include "llvm/Analysis/TargetLibraryInfo.h"
+#include "llvm/Analysis/TargetTransformInfo.h"
+#include "llvm/IR/DataLayout.h"
+#include "llvm/IR/LegacyPassNameParser.h"
 #include "llvm/IR/LegacyPassManager.h"
 #include "llvm/IR/Module.h"
-#include "llvm/IR/DataLayout.h"
-#include "llvm/ADT/Triple.h"
+#include "llvm/IR/Verifier.h"
+#include "llvm/LinkAllPasses.h"
+#include "llvm/Support/CommandLine.h"
+#include "llvm/Target/TargetMachine.h"
+#include "llvm/Transforms/Instrumentation.h"
+#if LDC_LLVM_VER >= 800
+#include "llvm/Transforms/Instrumentation/MemorySanitizer.h"
+#include "llvm/Transforms/Instrumentation/ThreadSanitizer.h"
+#endif
+#include "llvm/Transforms/IPO.h"
+#include "llvm/Transforms/IPO/PassManagerBuilder.h"
+
 #if LDC_LLVM_VER >= 400
 #include "llvm/Analysis/InlineCost.h"
 #endif
-#include "llvm/Analysis/TargetTransformInfo.h"
-#include "llvm/IR/Verifier.h"
-#include "llvm/Analysis/TargetLibraryInfo.h"
-#include "llvm/Target/TargetMachine.h"
-#include "llvm/Support/CommandLine.h"
-#include "llvm/IR/LegacyPassNameParser.h"
-#include "llvm/Transforms/Instrumentation.h"
-#include "llvm/Transforms/IPO.h"
-#include "llvm/Transforms/IPO/PassManagerBuilder.h"
 
 extern llvm::TargetMachine *gTargetMachine;
 using namespace llvm;
@@ -121,11 +126,20 @@ bool willCrossModuleInline() {
   return enableCrossModuleInlining == llvm::cl::BOU_TRUE;
 }
 
+#if LDC_LLVM_VER >= 800
+llvm::FramePointer::FP whichFramePointersToEmit() {
+  if (auto option = opts::framePointerUsage())
+    return *option;
+  return isOptimizationEnabled() ? llvm::FramePointer::None
+                                 : llvm::FramePointer::All;
+}
+#else
 bool willEliminateFramePointer() {
   const llvm::cl::boolOrDefault disableFPElimEnum = opts::disableFPElim();
   return disableFPElimEnum == llvm::cl::BOU_FALSE ||
          (disableFPElimEnum == llvm::cl::BOU_UNSET && isOptimizationEnabled());
 }
+#endif
 
 bool isOptimizationEnabled() { return optimizeLevel != 0; }
 
@@ -178,7 +192,11 @@ static void addAddressSanitizerPasses(const PassManagerBuilder &Builder,
 
 static void addMemorySanitizerPass(const PassManagerBuilder &Builder,
                                    PassManagerBase &PM) {
+#if LDC_LLVM_VER >= 800
+  PM.add(createMemorySanitizerLegacyPassPass());
+#else
   PM.add(createMemorySanitizerPass());
+#endif
 
   // MemorySanitizer inserts complex instrumentation that mostly follows
   // the logic of the original code, but operates on "shadow" values.
@@ -195,7 +213,11 @@ static void addMemorySanitizerPass(const PassManagerBuilder &Builder,
 
 static void addThreadSanitizerPass(const PassManagerBuilder &Builder,
                                    PassManagerBase &PM) {
+#if LDC_LLVM_VER >= 800
+  PM.add(createThreadSanitizerLegacyPassPass());
+#else
   PM.add(createThreadSanitizerPass());
+#endif
 }
 
 static void addSanitizerCoveragePass(const PassManagerBuilder &Builder,
@@ -214,11 +236,7 @@ static void addPGOPasses(PassManagerBuilder &builder,
     options.NoRedZone = global.params.disableRedZone;
     if (global.params.datafileInstrProf)
       options.InstrProfileOutput = global.params.datafileInstrProf;
-#if LDC_LLVM_VER >= 309
     mpm.add(createInstrProfilingLegacyPass(options));
-#else
-    mpm.add(createInstrProfilingPass(options));
-#endif
   } else if (opts::isUsingASTBasedPGOProfile()) {
 // We are generating code with PGO profile information available.
 #if LDC_LLVM_VER >= 500
@@ -228,7 +246,6 @@ static void addPGOPasses(PassManagerBuilder &builder,
     }
 #endif
   }
-#if LDC_LLVM_VER >= 309
   else if (opts::isInstrumentingForIRBasedPGO()) {
 #if LDC_LLVM_VER >= 400
     builder.EnablePGOInstrGen = true;
@@ -237,7 +254,6 @@ static void addPGOPasses(PassManagerBuilder &builder,
   } else if (opts::isUsingIRBasedPGOProfile()) {
     builder.PGOInstrUse = global.params.datafileInstrProf;
   }
-#endif
 }
 
 /**
@@ -257,10 +273,8 @@ static void addOptimizationPasses(legacy::PassManagerBase &mpm,
   PassManagerBuilder builder;
   builder.OptLevel = optLevel;
   builder.SizeLevel = sizeLevel;
-#if LDC_LLVM_VER >= 309
   builder.PrepareForLTO = opts::isUsingLTO();
   builder.PrepareForThinLTO = opts::isUsingThinLTO();
-#endif
 
   if (willInline()) {
 #if LDC_LLVM_VER >= 400
