@@ -26,8 +26,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef LDC_GEN_RECURSIVEVISITOR_H
-#define LDC_GEN_RECURSIVEVISITOR_H
+#pragma once
 
 #include "dmd/attrib.h"
 #include "dmd/declaration.h"
@@ -98,8 +97,8 @@ public:
     recurse(stmt->_body);
   }
 
-  void visit(OnScopeStatement *stmt) override {
-    stmt->error("Internal Compiler Error: OnScopeStatement should have been "
+  void visit(ScopeGuardStatement *stmt) override {
+    stmt->error("Internal Compiler Error: ScopeGuardStatement should have been "
                 "lowered by frontend.");
     fatal();
   }
@@ -337,7 +336,7 @@ public:
         recurse(stmt->_body);
   }
 
-  void visit(OnScopeStatement *stmt) override {
+  void visit(ScopeGuardStatement *stmt) override {
     call_visitor(stmt) || recurse(stmt->statement);
   }
 
@@ -476,5 +475,3 @@ public:
   void visit(Initializer *init) override { call_visitor(init); }
   void visit(Dsymbol *init) override {}
 };
-
-#endif // LDC_GEN_RECURSIVEVISITOR_H
