@@ -7,8 +7,19 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "attrib.h"
-#include "enum.h"
+#include "dmd/attrib.h"
+#include "dmd/ctfe.h"
+#include "dmd/enum.h"
+#include "dmd/hdrgen.h"
+#include "dmd/id.h"
+#include "dmd/identifier.h"
+#include "dmd/init.h"
+#include "dmd/ldcbindings.h"
+#include "dmd/module.h"
+#include "dmd/mtype.h"
+#include "dmd/root/port.h"
+#include "dmd/root/rmem.h"
+#include "dmd/template.h"
 #include "gen/aa.h"
 #include "gen/abi.h"
 #include "gen/arrays.h"
@@ -34,18 +45,9 @@
 #include "gen/tollvm.h"
 #include "gen/typinf.h"
 #include "gen/warnings.h"
-#include "hdrgen.h"
-#include "id.h"
-#include "init.h"
 #include "ir/irfunction.h"
 #include "ir/irtypeclass.h"
 #include "ir/irtypestruct.h"
-#include "ldcbindings.h"
-#include "module.h"
-#include "mtype.h"
-#include "port.h"
-#include "rmem.h"
-#include "template.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/ManagedStatic.h"
 #include <fstream>
@@ -53,9 +55,6 @@
 #include <stack>
 #include <stdio.h>
 
-// Needs other includes.
-#include "ctfe.h"
-
 llvm::cl::opt<bool> checkPrintf(
     "check-printf-calls", llvm::cl::ZeroOrMore,
     llvm::cl::desc("Validate printf call format strings against arguments"));
@@ -398,11 +397,7 @@ public:
       }
       gvar = new llvm::GlobalVariable(gIR->module, at, true, _linkage, _init,
                                       ".str");
-#if LDC_LLVM_VER >= 309
       gvar->setUnnamedAddr(llvm::GlobalValue::UnnamedAddr::Global);
-#else
-      gvar->setUnnamedAddr(true);
-#endif
       (*stringLiteralCache)[key] = gvar;
     }
 
@@ -415,7 +410,7 @@ public:
     if (dtype->ty == Tarray) {
       LLConstant *clen =
           LLConstantInt::get(DtoSize_t(), e->numberOfCodeUnits(), false);
-      result = new DImValue(e->type, DtoConstSlice(clen, arrptr, dtype));
+      result = new DSliceValue(e->type, DtoConstSlice(clen, arrptr, dtype));
     } else if (dtype->ty == Tsarray) {
       LLType *dstType =
           getPtrToType(LLArrayType::get(ct, e->numberOfCodeUnits()));
@@ -507,10 +502,30 @@ public:
       }
     };
 
-    // try to construct the lhs in-place
-    if (lhs->isLVal() && e->op == TOKconstruct &&
-        toInPlaceConstruction(lhs->isLVal(), e->e2)) {
-      return;
+    // Try to construct the lhs in-place.
+    if (lhs->isLVal() && (e->op == TOKconstruct)) {
+      Logger::println("attempting in-place construction");
+      if (toInPlaceConstruction(lhs->isLVal(), e->e2))
+        return;
+    }
+
+    // Try to assign to the lhs in-place.
+    // This extra complication at -O0 is to prevent excessive stack space usage
+    // when assigning to large structs.
+    // Note: If the assignment is non-trivial, a CallExp to opAssign is
+    // generated by the frontend instead of this AssignExp. The in-place
+    // construction is not valid if the rhs is not a literal (consider for
+    // example `a = foo(a)`), but also not if the rhs contains non-constant
+    // elements (consider for example `a = [0, a[0], 2]` or `a = [0, i, 2]`
+    // where `i` is a ref variable aliasing with a).
+    // Be conservative with this optimization for now: only do the optimization
+    // for struct `.init` assignment.
+    if (lhs->isLVal() && (e->op == TOKassign) &&
+        ((e->e2->op == TOKstructliteral) &&
+         static_cast<StructLiteralExp *>(e->e2)->useStaticInit)) {
+      Logger::println("attempting in-place assignment");
+      if (toInPlaceConstruction(lhs->isLVal(), e->e2))
+        return;
     }
 
     DValue *r = toElem(e->e2);
@@ -736,7 +751,7 @@ public:
 
       // as requested by bearophile, see if it's a C printf call and that it's
       // valid.
-      if (global.params.warnings && checkPrintf) {
+      if (global.params.warnings != DIAGNOSTICoff && checkPrintf) {
         if (fndecl->linkage == LINKc &&
             strcmp(fndecl->ident->toChars(), "printf") == 0) {
           warnInvalidPrintfCall(e->loc, (*e->arguments)[0], e->arguments->dim);
@@ -1032,8 +1047,8 @@ public:
     const auto ident = p->func()->decl->ident;
     if (ident == Id::ensure || ident == Id::require) {
       Logger::println("contract this exp");
-      LLValue *v = DtoBitCast(p->func()->nestArg, DtoType(e->type));
-      result = new DImValue(e->type, v);
+      LLValue *v = p->func()->nestArg; // thisptr lvalue
+      result = new DLValue(e->type, DtoBitCast(v, DtoPtrToType(e->type)));
     } else if (vd->toParent2() != p->func()->decl) {
       Logger::println("nested this exp");
       result = DtoNestedVariable(e->loc, e->type, vd, e->type->ty == Tstruct);
@@ -1593,9 +1608,7 @@ public:
       } else if (e->e1->op == TOKvar) {
         if (auto vd = static_cast<VarExp *>(e->e1)->var->isVarDeclaration()) {
           if (vd->onstack) {
-            assert(vd->scopeClassType);
-            const auto cd = vd->scopeClassType->sym->isClassDeclaration();
-            DtoFinalizeScopeClass(e->loc, DtoRVal(dval), cd);
+            DtoFinalizeScopeClass(e->loc, DtoRVal(dval), vd->onstackWithDtor);
             onstack = true;
           }
         }
@@ -1690,7 +1703,7 @@ public:
     p->scope() = IRScope(passedbb);
 
     // class/struct invariants
-    if (!global.params.useInvariants)
+    if (global.params.useInvariants != CHECKENABLEon)
       return;
     if (condty->ty == Tclass) {
       const auto sym = static_cast<TypeClass *>(condty)->sym;
@@ -1961,6 +1974,10 @@ public:
       assert(lv->getType() == rv->getType());
       eval = (e->op == TOKidentity) ? p->ir->CreateICmpEQ(lv, rv)
                                     : p->ir->CreateICmpNE(lv, rv);
+      if (t1->ty == Tvector) {
+        eval = mergeVectorEquals(eval,
+                                 e->op == TOKidentity ? TOKequal : TOKnotequal);
+      }
     }
     result = new DImValue(e->type, eval);
   }
