@@ -8,12 +8,14 @@
 //===----------------------------------------------------------------------===//
 
 #include "gen/tollvm.h"
-#include "aggregate.h"
-#include "declaration.h"
-#include "dsymbol.h"
-#include "id.h"
-#include "init.h"
-#include "module.h"
+
+#include "dmd/aggregate.h"
+#include "dmd/declaration.h"
+#include "dmd/dsymbol.h"
+#include "dmd/expression.h"
+#include "dmd/id.h"
+#include "dmd/init.h"
+#include "dmd/module.h"
 #include "gen/abi.h"
 #include "gen/arrays.h"
 #include "gen/classes.h"
@@ -50,12 +52,12 @@ bool DtoIsReturnInArg(CallExp *ce) {
   return false;
 }
 
-LLAttribute DtoShouldExtend(Type *type) {
+void DtoAddExtendAttr(Type *type, llvm::AttrBuilder &attrs) {
   type = type->toBasetype();
   if (type->isintegral() && type->ty != Tvector && type->size() <= 2) {
-    return type->isunsigned() ? LLAttribute::ZExt : LLAttribute::SExt;
+    attrs.addAttribute(type->isunsigned() ? LLAttribute::ZExt
+                                          : LLAttribute::SExt);
   }
-  return LLAttribute::None;
 }
 
 LLType *DtoType(Type *t) {
@@ -185,17 +187,11 @@ LLType *DtoMemType(Type *t) { return i1ToI8(voidToI8(DtoType(t))); }
 LLPointerType *DtoPtrToType(Type *t) { return DtoMemType(t)->getPointerTo(); }
 
 LLType *voidToI8(LLType *t) {
-  if (t == LLType::getVoidTy(gIR->context())) {
-    return LLType::getInt8Ty(gIR->context());
-  }
-  return t;
+  return t->isVoidTy() ? LLType::getInt8Ty(t->getContext()) : t;
 }
 
 LLType *i1ToI8(LLType *t) {
-  if (t == LLType::getInt1Ty(gIR->context())) {
-    return LLType::getInt8Ty(gIR->context());
-  }
-  return t;
+  return t->isIntegerTy(1) ? LLType::getInt8Ty(t->getContext()) : t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -433,11 +429,7 @@ LLConstant *DtoConstCString(const char *str) {
     gvar = new llvm::GlobalVariable(gIR->module, init->getType(), true,
                                     llvm::GlobalValue::PrivateLinkage, init,
                                     ".str");
-#if LDC_LLVM_VER >= 309
     gvar->setUnnamedAddr(llvm::GlobalValue::UnnamedAddr::Global);
-#else
-    gvar->setUnnamedAddr(true);
-#endif
     gIR->stringLiteral1ByteCache[s] = gvar;
   }
 
