@@ -13,16 +13,15 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef LDC_GEN_TOLLVM_H
-#define LDC_GEN_TOLLVM_H
+#pragma once
 
-#include "attrib.h"
-#include "declaration.h"
-#include "tokens.h"
-#include "mtype.h"
+#include "dmd/attrib.h"
+#include "dmd/declaration.h"
+#include "dmd/tokens.h"
+#include "dmd/mtype.h"
+#include "gen/attributes.h"
 #include "gen/llvm.h"
 #include "gen/structs.h"
-#include "gen/attributes.h"
 
 // D->LLVM type handling stuff
 
@@ -55,8 +54,8 @@ bool DtoIsInMemoryOnly(Type *type);
 // address as additional parameter to the callee, which will set it up.
 bool DtoIsReturnInArg(CallExp *ce);
 
-// should argument be zero or sign extended
-LLAttribute DtoShouldExtend(Type *type);
+// Adds an appropriate attribute if the type should be zero or sign extended.
+void DtoAddExtendAttr(Type *type, llvm::AttrBuilder &attrs);
 
 // tuple helper
 // takes a arguments list and makes a struct type out of them
@@ -204,5 +203,3 @@ void DtoMemCpy(LLValue *dst, LLValue *src, bool withPadding = false,
  * Generates a call to C memcmp.
  */
 LLValue *DtoMemCmp(LLValue *lhs, LLValue *rhs, LLValue *nbytes);
-
-#endif // LDC_GEN_TOLLVM_H
