@@ -22,19 +22,20 @@
 
 #include "gen/typinf.h"
 
-#include "aggregate.h"
-#include "attrib.h"
-#include "declaration.h"
-#include "enum.h"
-#include "expression.h"
-#include "id.h"
-#include "import.h"
-#include "init.h"
-#include "mars.h"
-#include "module.h"
-#include "mtype.h"
-#include "scope.h"
-#include "template.h"
+#include "dmd/aggregate.h"
+#include "dmd/attrib.h"
+#include "dmd/declaration.h"
+#include "dmd/enum.h"
+#include "dmd/expression.h"
+#include "dmd/id.h"
+#include "dmd/import.h"
+#include "dmd/init.h"
+#include "dmd/mangle.h"
+#include "dmd/mars.h"
+#include "dmd/module.h"
+#include "dmd/mtype.h"
+#include "dmd/scope.h"
+#include "dmd/template.h"
 #include "gen/arrays.h"
 #include "gen/classes.h"
 #include "gen/irstate.h"
@@ -44,15 +45,16 @@
 #include "gen/logger.h"
 #include "gen/mangling.h"
 #include "gen/metadata.h"
+#include "gen/pragma.h"
 #include "gen/rttibuilder.h"
 #include "gen/runtime.h"
 #include "gen/structs.h"
 #include "gen/tollvm.h"
 #include "ir/irtype.h"
+#include <ir/irtypeclass.h>
 #include "ir/irvar.h"
 #include <cassert>
 #include <cstdio>
-#include <ir/irtypeclass.h>
 
 FuncDeclaration *search_toString(StructDeclaration *sd);
 
@@ -645,10 +647,19 @@ void TypeInfoDeclaration_codegen(TypeInfoDeclaration *decl, IRState *p) {
   emitTypeMetadata(decl);
 
   // check if the definition can be elided
+  Type *forType = decl->tinfo;
   if (!global.params.useTypeInfo || !Type::dtypeinfo ||
-      isSpeculativeType(decl->tinfo) || builtinTypeInfo(decl->tinfo)) {
+      isSpeculativeType(forType) || builtinTypeInfo(forType)) {
     return;
   }
+  if (auto forStructType = forType->isTypeStruct()) {
+    if (forStructType->sym->llvmInternal == LLVMno_typeinfo)
+      return;
+  }
+  if (auto forClassType = forType->isTypeClass()) {
+    if (forClassType->sym->llvmInternal == LLVMno_typeinfo)
+      return;
+  }
 
   // define the TypeInfo global
   LLVMDefineVisitor v(gvar);
