@@ -7,33 +7,35 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "llvm/IR/Constants.h"
-#include "llvm/IR/DerivedTypes.h"
-#include "llvm/ADT/SmallString.h"
-#ifndef NDEBUG
-#include "llvm/Support/raw_ostream.h"
-#endif
-
-#include "aggregate.h"
-#include "declaration.h"
-#include "hdrgen.h" // for parametersTypeToChars()
-#include "mtype.h"
-#include "target.h"
+#include "dmd/aggregate.h"
+#include "dmd/declaration.h"
+#include "dmd/hdrgen.h" // for parametersTypeToChars()
+#include "dmd/identifier.h"
+#include "dmd/mangle.h"
+#include "dmd/mtype.h"
+#include "dmd/target.h"
+#include "gen/abi.h"
+#include "gen/arrays.h"
 #include "gen/funcgenstate.h"
+#include "gen/functions.h"
 #include "gen/irstate.h"
 #include "gen/logger.h"
-#include "gen/tollvm.h"
 #include "gen/llvmhelpers.h"
-#include "gen/arrays.h"
-#include "gen/metadata.h"
-#include "gen/runtime.h"
-#include "gen/functions.h"
-#include "gen/abi.h"
 #include "gen/mangling.h"
-
+#include "gen/metadata.h"
+#include "gen/pragma.h"
+#include "gen/runtime.h"
+#include "gen/tollvm.h"
 #include "ir/iraggr.h"
 #include "ir/irfunction.h"
 #include "ir/irtypeclass.h"
+#include "llvm/ADT/SmallString.h"
+#include "llvm/IR/Constants.h"
+#include "llvm/IR/DerivedTypes.h"
+
+#ifndef NDEBUG
+#include "llvm/Support/raw_ostream.h"
+#endif
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -59,6 +61,13 @@ LLGlobalVariable *IrAggr::getVtblSymbol() {
 
 //////////////////////////////////////////////////////////////////////////////
 
+bool IrAggr::suppressTypeInfo() const {
+  return !global.params.useTypeInfo || !Type::dtypeinfo ||
+         aggrdecl->llvmInternal == LLVMno_typeinfo;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 LLGlobalVariable *IrAggr::getClassInfoSymbol() {
   if (classInfo) {
     return classInfo;
@@ -164,7 +173,7 @@ LLConstant *IrAggr::getVtblInit() {
   // start with the classinfo
   llvm::Constant *c;
   if (!cd->isCPPclass()) {
-    if (global.params.useTypeInfo && Type::dtypeinfo) {
+    if (!suppressTypeInfo()) {
       c = getClassInfoSymbol();
       c = DtoBitCast(c, voidPtrType);
     } else {
@@ -192,6 +201,11 @@ LLConstant *IrAggr::getVtblInit() {
       if (fd->inferRetType && !fd->type->nextOf()) {
         Logger::println("Running late functionSemantic to infer return type.");
         if (!fd->functionSemantic()) {
+          if (fd->semantic3Errors) {
+            Logger::println("functionSemantic failed; using null for vtbl entry.");
+            constants.push_back(getNullValue(voidPtrType));
+            continue;
+          }
           fd->error("failed to infer return type for vtbl initializer");
           fatal();
         }
@@ -221,9 +235,8 @@ LLConstant *IrAggr::getVtblInit() {
               cd->error("use of `%s%s` is hidden by `%s`; use `alias %s = "
                         "%s.%s;` to introduce base class overload set",
                         fd->toPrettyChars(),
-                        parametersTypeToChars(tf->parameters, tf->varargs),
-                        cd->toChars(), fd->toChars(), fd->parent->toChars(),
-                        fd->toChars());
+                        parametersTypeToChars(tf->parameterList), cd->toChars(),
+                        fd->toChars(), fd->parent->toChars(), fd->toChars());
             } else {
               cd->error("use of `%s` is hidden by `%s`", fd->toPrettyChars(),
                         cd->toChars());
@@ -317,7 +330,7 @@ void IrAggr::defineInterfaceVtbl(BaseClass *b, bool new_instance,
   const auto voidPtrTy = getVoidPtrType();
 
   if (!b->sym->isCPPinterface()) { // skip interface info for CPP interfaces
-    if (global.params.useTypeInfo && Type::dtypeinfo) {
+    if (!suppressTypeInfo()) {
       // index into the interfaces array
       llvm::Constant *idxs[2] = {DtoConstSize_t(0),
                                  DtoConstSize_t(interfaces_index)};
@@ -386,13 +399,8 @@ void IrAggr::defineInterfaceVtbl(BaseClass *b, bool new_instance,
       setLinkage(lwc, thunk);
       thunk->copyAttributesFrom(callee);
 
-// Thunks themselves don't have an identity, only the target
-// function has.
-#if LDC_LLVM_VER >= 309
+      // Thunks themselves don't have an identity, only the target function has.
       thunk->setUnnamedAddr(llvm::GlobalValue::UnnamedAddr::Global);
-#else
-      thunk->setUnnamedAddr(true);
-#endif
 
       // thunks don't need exception handling themselves
       thunk->setPersonalityFn(nullptr);
@@ -414,6 +422,7 @@ void IrAggr::defineInterfaceVtbl(BaseClass *b, bool new_instance,
       gIR->funcGenStates.emplace_back(new FuncGenState(*thunkFunc, *gIR));
 
       // debug info
+      thunkFunc->diSubprogram = nullptr;
       thunkFunc->diSubprogram = gIR->DBuilder.EmitThunk(thunk, thunkFd);
 
       // create entry and end blocks
